{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                Visual parameters editor               }
{*******************************************************}
unit ParamEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, ComCtrls, Probes, ProbeParams, Session, math,
  Placemnt, DynamicControls, TemplateFrame, ActnList, ToolWin;

type
  TfmEditParams = class(TForm)
    pnParams: TGroupBox;
    Panel2: TPanel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Splitter1: TSplitter;
    ScrollBox1: TScrollBox;
    FormStorage1: TFormStorage;
    frTemplate: TfrTemplate;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ActionList1: TActionList;
    acShowInputParams: TAction;
    acShowOutputParams: TAction;
    procedure ScrollBox1Resize(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure acShowInputParamsExecute(Sender: TObject);
    procedure acShowOutputParamsExecute(Sender: TObject);
    procedure frTemplatetvTemplateMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    FTemplate: TSessionTemplate;
    FOnExecute: Boolean;
    FControlList: TList;
    FInsertProcess: Boolean;
    procedure FillPanel(Panel: TScrollingWinControl; AllParams: Boolean);
    procedure PanelToParams(Panel: TWinControl);
    procedure TreeLocate(TemplateItem: TTemplateItem; Param: TParam);
    procedure ControlEnter(Sender: TObject);
    procedure OrderControl(Panel: TScrollingWinControl; Control: TControl;
      Lbl: TLabel; var LastX, LastY, LastHeight: Integer);
    procedure ReorderControls(Panel: TScrollingWinControl);
    function GetControl(Panel: TWinControl; TemplateItem: TTemplateItem;
      Param: TParam; Lbl: Boolean): TControl;
    function NeedShowParam(Item: TTemplateItem; Param: TParam): Boolean;
  end;

function EditParams(Template: TSessionTemplate; OnExecute: Boolean): boolean;

implementation
{$R *.DFM}

const
  Margin = 5;

function EditParams(Template: TSessionTemplate; OnExecute: Boolean): boolean;
begin
  result:=False;
  with TFmEditParams.Create(Application) do
  try
    FTemplate:=Template;
    frTemplate.Template:=FTemplate;
    frTemplate.TemplateToView;
    FOnExecute:=OnExecute;
    FillPanel(ScrollBox1, False);
    if ShowModal = mrOk then result:=True;
  finally
    Free;
  end;
end;

procedure TfmEditParams.ControlEnter(Sender: TObject);
begin
  TreeLocate(IDCFromControl(TWinControl(Sender)).GetTemplateItem,
    IDCFromControl(TWinControl(Sender)).GetParam);
end;

procedure TfmEditParams.OrderControl(Panel: TScrollingWinControl; Control: TControl; Lbl: TLabel;
  var LastX, LastY, LastHeight: Integer);
var
  Width: Integer;
begin
  inc(LastX, Margin);
  Width:=Panel.Width;
  if Panel.HorzScrollBar.Visible then Width:=Width-GetSystemMetrics(SM_CXVSCROLL);
  if LastX+Max(Lbl.Width, Control.Width)>=Width then
  begin
    inc(LastY, LastHeight);
    LastX:=Margin;
  end;
  LastHeight:=Margin+Lbl.Height+Control.Height;
  Lbl.Left:=LastX;
  Lbl.Top:=LastY;
  Control.Left:=LastX;
  Control.Top:=LastY+Lbl.Height;
  LastX:=LastX+Max(Lbl.Width, Control.Width);
end;

Type
  THackWinControl = class(TWinControl);

procedure TfmEditParams.FillPanel(Panel: TScrollingWinControl; AllParams: Boolean);
var
  LastX, LastY, LastHeight: Integer;
  FFocus: boolean;

  procedure RemoveAll;
  var
    i: integer;
    Control: TControl;
  begin
    for i:=FControlList.Count-1 downto 0 do
    begin
      Control:=TControl(FControlList[i]);
      Control.Free;
    end;
    FControlList.Clear;
  end;

  procedure GetParentParams(Item: TTemplateItem; Strings: TStrings);

    procedure OneItem(Item: TTemplateItem; Strings: TStrings);
    var
      i: integer;
    begin
      if Item.OwnerItem<>nil then
      begin
        for i:=0 to Item.OwnerItem.Params.Count-1 do
          Strings.AddObject(FTemplate.FullParamName(Item.OwnerItem.Params[i]),
            Item.OwnerItem.Params[i]);
        OneItem(Item.OwnerItem, Strings);
      end;
    end;

  begin
    OneItem(Item, Strings)
  end;

  function CreateControl(Item: TTemplateItem; Param: TParam): TWinControl;
  var
    ParamSource: TParamSource;
    Value: String;
  begin
    ParamSource:=Item.ParamSources.FindByParamName(Param.Name);
    if FOnExecute then
      result:=TDynamicEdit.Create(Self)
    else
      result:=TDynamicCombo.Create(Self);
    Panel.InsertControl(result);
    THackWinControl(result).OnEnter:=ControlEnter;

    // Defaul values
    if ParamSource<>nil then
      Value:=ParamSource.DefaultValue else Value:=Param.AsString;
    IDCFromControl(result).SetProps(Item, Param, Value);
    if not FOnExecute then
    With TDynamicCombo(result) do
    begin
      GetParentParams(Item, Items);
      if (ParamSource<>nil) then
      begin
        ItemIndex:=Items.IndexOfObject(ParamSource.SourceParam);
        if ParamSource.DefaultValue<>'' then Text:=ParamSource.DefaultValue;
      end;
    end;
  end;

  procedure Insert(Control: TControl; const Caption: string);
  var
    FLabel: TLabel;
  begin
    FLabel:=TDynamicLabel.Create(Self);
    IDCFromControl(FLabel).SetProps(
      IDCFromControl(Control).GetTemplateItem, IDCFromControl(Control).GetParam, Caption);
    Panel.InsertControl(FLabel);
    FControlList.Add(FLabel);
    FControlList.Add(Control);
    OrderControl(Panel, Control, FLabel, LastX, LastY, LastHeight);
  end;


  procedure OneItem(Item: TTemplateItem);
  var
    i: integer;
    Control: TControl;
    CaptionText: String;
  begin
    if Item<>FTemplate.RootItem then
    begin
      Control:=TDynamicShape.Create(Self);
      Panel.InsertControl(Control);
      Control.Width:=Panel.Width-2*Margin;
      Control.Left:=Margin;
      Control.Height:=3;
      IDCFromControl(Control).SetProps(Item, nil, '');
      Insert(Control, Item.Alias);
    end;

    for i:=0 to Item.Params.Count-1 do
    With Item.Params[i] do
    begin
      if NeedShowParam(Item, Item.Params[i]) then
      begin
        Control:=CreateControl(Item, Item.Params[i]);
        CaptionText:=Name;
        if poRequired in ParamOptions then CaptionText:=CaptionText+' (*)';
        Insert(Control, CaptionText);
        if not FFocus and (Control is TWinControl) then
        begin
          ActiveControl:=TWinControl(Control);
          FFocus:=True;
        end;
      end;
    end;

    for i:=0 to Item.ChildItems.Count-1 do
      OneItem(Item.ChildItems[i]);
  end;

begin
  if FControlList=nil then FControlList:=TList.Create else RemoveAll;
  LastX:=0; LastY:=Margin; LastHeight:=0;
  FFocus:=False;
  FInsertProcess:=True;
  Panel.DisableAutoRange;
  try
    OneItem(FTemplate.RootItem);
  finally
    Panel.EnableAutoRange;
    FInsertProcess:=False;
  end;
end;

procedure TfmEditParams.ReorderControls(Panel: TScrollingWinControl);
var
  LastX, LastY, LastHeight: Integer;

  procedure OneItem(Item: TTemplateItem);
  var
    i: integer;
    Control: TControl;
  begin
    if Item<>FTemplate.RootItem then
    begin
      Control:=GetControl(Panel, Item, nil, False);
      if Panel.HorzScrollBar.Visible then
        Control.Width:=Panel.Width-2*Margin-GetSystemMetrics(SM_CXVSCROLL) else
        Control.Width:=Panel.Width-2*Margin;
      OrderControl(Panel, Control,
        GetControl(Panel, Item, nil, True) as TLabel, LastX, LastY, LastHeight);
    end;

    for i:=0 to Item.Params.Count-1 do
      if NeedShowParam(Item, Item.Params[i]) then
        OrderControl(Panel, GetControl(Panel, Item, Item.Params[i], False),
          GetControl(Panel, Item, Item.Params[i], True) as TLabel, LastX, LastY, LastHeight);

    for i:=0 to Item.ChildItems.Count-1 do OneItem(Item.ChildItems[i]);
  end;

begin
  LastX:=0; LastY:=Margin; LastHeight:=0;
  OneItem(FTemplate.RootItem);
end;

procedure TfmEditParams.TreeLocate(TemplateItem: TTemplateItem; Param: TParam);
var
  i: integer;
begin
  for i:=0 to frTemplate.tvTemplate.Items.Count-1 do
    if ((Param <> nil) and (frTemplate.tvTemplate.Items[i].Data = Param))
     or ((TemplateItem <> nil) and (frTemplate.tvTemplate.Items[i].Data = TemplateItem))then
    begin
      frTemplate.tvTemplate.Items[i].Focused:=true;
      frTemplate.tvTemplate.Items[i].Selected:=true;
    end;
end;

procedure TfmEditParams.ScrollBox1Resize(Sender: TObject);
begin
  if not FInsertProcess then ReorderControls(ScrollBox1);
end;

function TfmEditParams.GetControl(Panel: TWinControl; TemplateItem: TTemplateItem;
  Param: TParam; Lbl: Boolean): TControl;

  function SameComp(Control: TControl): Boolean;
  begin
    result:=(lbl and (Control is TLabel)) or (not lbl and not (Control is TLabel));
  end;
var
  i: integer;
begin

  result:=nil;
  for i:=0 to FControlList.Count-1 do
    if SameComp(FControlList[i]) and
      ((TemplateItem = nil) or
      (IDCFromControl(FControlList[i]).GetTemplateItem = TemplateItem)) and
      (IDCFromControl(FControlList[i]).GetParam = Param) then
    begin
      result:=FControlList[i];
      Break;
    end;
  Assert(result<>nil);
end;


procedure TfmEditParams.PanelToParams(Panel: TWinControl);

  procedure OneItem(Item: TTemplateItem);
  var
    i: integer;
    Control: TControl;
    Value: String;
    SourceParam: TParam;
    Index: Integer;
  begin
    for i:=0 to Item.Params.Count-1 do
      if NeedShowParam(Item, Item.Params[i]) then
      begin
        Control:=GetControl(Panel, Item, Item.Params[i], False);
        Value:=IDCFromControl(Control).GetValue;
        SourceParam:=nil;
        if (Control is TComboBox) then
        begin
          Index:=TComboBox(Control).Items.IndexOf(Value);
          if Index<>-1 then
            SourceParam:=TParam(TComboBox(Control).Items.Objects[Index]);
        end;
        if SourceParam<>nil then
          Item.ParamSources.NewItem(Item.Params[i].Name, SourceParam, '') else
          Item.ParamSources.NewItem(Item.Params[i].Name, nil, Value);
      end;
    for i:=0 to Item.ChildItems.Count-1 do
      OneItem(Item.ChildItems[i]);
  end;

begin
  OneItem(FTemplate.RootItem);
end;

procedure TfmEditParams.BitBtn1Click(Sender: TObject);
begin
  PanelToParams(ScrollBox1);
end;

procedure TfmEditParams.acShowInputParamsExecute(Sender: TObject);
begin
  frTemplate.ShowInputParams:=not frTemplate.ShowInputParams;
  frTemplate.TemplateToView;
end;

procedure TfmEditParams.acShowOutputParamsExecute(Sender: TObject);
begin
  frTemplate.ShowOutputParams:=not frTemplate.ShowOutputParams;
  frTemplate.TemplateToView;
end;

procedure TfmEditParams.frTemplatetvTemplateMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
//  Control: TControl;
  Param: TParam;
  i: integer;
  Node: TTreeNode;
  TemplateItem: TTemplateItem;
begin
  if Button = mbRight then
  begin
    Node:=frTemplate.tvTemplate.GetNodeAt(X,Y);
    if (Node<>nil) and (Node.Data<>nil) then
    begin
      if TObject(Node.Data) is TTemplateItem then
      With TTemplateItem(Node.Data) do
      begin
        // Find first input parameter
        Param:=nil;
        for i:=0 to Params.Count do
          if Params[i].ParamType = ptInput then
          begin
            Param:=Params[i];
            break;
          end;
        if Param<>nil then
          ActiveControl:=GetControl(ScrollBox1, Node.Data, Param, False) as TWinControl;
      end else
        if (TObject(Node.Data) is TParam) and (TParam(Node.Data).ParamType = ptInput) then
        begin
          TemplateItem:=GetParamOwner(TParam(Node.Data));
          if NeedShowParam(TemplateItem, TParam(Node.Data)) then
            ActiveControl:=GetControl(ScrollBox1, nil, Node.Data, False) as TWinControl;
        end;
    end;
  end;
end;

function TfmEditParams.NeedShowParam(Item: TTemplateItem; Param: TParam): Boolean;
var
  ParamSource: TParamSource;
begin
  // Do not show in sesison whose params that has parent sources
  if FOnExecute then
    ParamSource:=Item.ParamSources.FindByParamName(Param.Name) else ParamSource:=nil;
  result:=(Param.ParamType = ptInput) and
    not (FOnExecute and (ParamSource<>nil) and (ParamSource.SourceParam <> nil));
end;

end.
