unit PluginAPI;
interface
uses windows;        

Type
// About string management
// If Buffer and BufSize parameter used and buffer is not large
// enough to contain the entire value, the function must fail and return
// the required size of the buffer

// Return probe description
  TGetProbeInfo = function(ProbeName: PChar; Description: PChar;
    var BufSize: Integer): Boolean; stdcall;

// Callback function for EnumProbeNames
  TProbeNameCallback = function(ProbeName: PChar; Data: pointer): Boolean; stdcall;

// Enumerate probes supported by this plugin
// if plugin is support only one probe it's not implement this
  TEnumProbeNames = procedure(cbProbeName: TProbeNameCallback; Data: Pointer); stdcall;

// Callback function for EnumProbeParams. Return information about probe parameter
// return value false - break enumeration
  TProbeParamCallback = function(ParamName: PChar; ParamType: Integer;
    Input: Boolean; Required: Boolean; DefaultValue: PChar; Data: Pointer): Boolean; stdcall;

// Enumerate probe params. If plugin support one probe, ProbeName may be nil
// Data parameter must be passed to callback function
  TEnumProbeParams = procedure(ProbeName: PChar; cbProbeParams: TProbeParamCallback;
    Data: Pointer); stdcall;

  TcbGetValue = function(ParamName: PChar; Data: Pointer; Value: PChar;
    var BufSize: Integer): Boolean; stdcall;

// Start probe. This funciton must return immediately and send all information to
// window specified by HWND parameter, Message specified by Msg parameter
// Input parameters may be acquired by cbParamValues
// Return probe handle, unique in this library
// Data parameter must be passed to callback function

  TStartProbe = function(ProbeName: PChar; cbParamValues: TcbGetValue;
    Msg: Integer; HWND: HWND; Data: Pointer): THandle; stdcall;

// Stop probe specified by ProbeHandle parameter, not return until probe is
// really finished
  TStopProbe = procedure(ProbeHandle: Integer); stdcall;


  TcbSetValue = function(ParamName: PChar; Data: Pointer; Value: PChar): Boolean; stdcall;

// Get current probe parameteres
// Called than DLL sends message to main program
  TGetCurrentValues = procedure(ProbeHandle: Integer;
    cbParamValues: TcbSetValue; Data: Pointer); stdcall;

// format of message sended to main program
// message must be sended by SendMessage not PostMessage
// Parameters must include two pseudo parameters 'ProbeFinished' and 'ProbeSuccess'
// hWND - specified by hwnd parametere of StartProbe
// Msg - specified by msg parametere of StartProbe
// WParam - probe handle

// exported function names
const
  efGetProbeInfo = 'GetProbeInfo';
  efEnumProbeParams = 'EnumProbeParams';
  efStartProbe = 'StartProbe';
  efStopProbe = 'StopProbe';
  efGetCurrentValues = 'GetCurrentValues';
  efEnumProbeNames = 'EnumProbeNames';

implementation

end.
