{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                Session view visual editor             }
{*******************************************************}
unit ViewEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ImgList, ActnList, ToolWin, TemplateFrame, ExtCtrls,
  Buttons, Session, ProbeParams, Placemnt;

type
  TfmViewEditor = class(TForm)
    frTemplate: TfrTemplate;
    ToolBar1: TToolBar;
    ActionList1: TActionList;
    acNewGroup: TAction;
    acDeleteGroup: TAction;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    Panel2: TPanel;
    Splitter1: TSplitter;
    pnView: TGroupBox;
    tvView: TTreeView;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    acViewInputParams: TAction;
    acViewOutputParams: TAction;
    Button1: TButton;
    FormStorage1: TFormStorage;
    acGroupUp: TAction;
    acGroupDown: TAction;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    procedure acViewInputParamsExecute(Sender: TObject);
    procedure acViewOutputParamsExecute(Sender: TObject);
    procedure acNewGroupExecute(Sender: TObject);
    procedure acDeleteGroupUpdate(Sender: TObject);
    procedure acDeleteGroupExecute(Sender: TObject);
    procedure tvViewEdited(Sender: TObject; Node: TTreeNode;
      var S: String);
    procedure tvViewEditing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure frTemplatetvTemplateMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure frTemplatetvTemplateDragOver(Sender, Source: TObject; X,
      Y: Integer; State: TDragState; var Accept: Boolean);
    procedure tvViewDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure tvViewDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure tvViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure tvViewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure frTemplatetvTemplateDragDrop(Sender, Source: TObject; X,
      Y: Integer);
    procedure acGroupUpUpdate(Sender: TObject);
    procedure acGroupDownUpdate(Sender: TObject);
    procedure acGroupUpExecute(Sender: TObject);
    procedure acGroupDownExecute(Sender: TObject);
  private
    FTemplate: TSessionTemplate;
    FDraggedNode: TTreeNode;
    function IsGroupNode(Node: TTreeNode): Boolean;
    function IsParamNode(Node: TTreeNode): Boolean;
    function GetGroupByNode(Node: TTreeNode): TParamGroup;
    function GetParamByNode(Node: TTreeNode; var Group: TParamGroup): TParam;
    function AddGroupToTree(Group: TParamGroup): TTreeNode;
    function AddParamToTree(GroupNode: TTreeNode; Param: TParam): TTreeNode;
    procedure DeleteNode(Node: TTreeNode);
  public
    procedure DisplayView;
  end;

function EditView(Template: TSessionTemplate): Boolean;

implementation
uses iphconsts, utils, contutils;
{$R *.DFM}

function EditView(Template: TSessionTemplate): Boolean;
begin
 with TfmViewEditor.Create(Application) do
 try
   FTemplate:=Template;
   frTemplate.Template:=FTemplate;
   frTemplate.ShowInputParams:=True;
   frTemplate.ShowOutputParams:=True;
   frTemplate.TemplateToView;
   DisplayView;
   result:=ShowModal = mrOk;
 finally
   free;
 end;
end;

procedure TfmViewEditor.acViewInputParamsExecute(Sender: TObject);
begin
  frTemplate.ShowInputParams:=not frTemplate.ShowInputParams;
  frTemplate.TemplateToView;
end;

procedure TfmViewEditor.acViewOutputParamsExecute(Sender: TObject);
begin
  frTemplate.ShowOutputParams:=not frTemplate.ShowOutputParams;
  frTemplate.TemplateToView;
end;

function TfmViewEditor.AddParamToTree(GroupNode: TTreeNode;
  Param: TParam): TTreeNode;
begin
  Result:=tvView.Items.AddChildObject(GroupNode,
    GetParamOwner(Param).Alias+'.'+Param.Name, Param);
  if Param.ParamType = ptInput then
    SetImageIndex(Result, 0) else SetImageIndex(Result, 1);
end;

function TfmViewEditor.AddGroupToTree(Group: TParamGroup): TTreeNode;
var
  i: Integer;
begin
  Result:=tvView.Items.AddObject(nil, Group.GroupName, Group);
  if Group.GroupName = grVisibleParams then
    SetImageIndex(result, 3) else SetImageIndex(result, 4);
  for i:=0 to Group.Count-1 do
    AddParamToTree(Result, Group[i]);
end;

procedure TfmViewEditor.DisplayView;
var
  i: integer;
begin
  tvView.Items.BeginUpdate;
  try
    tvView.Items.Clear;
    for i:=0 to FTemplate.View.Groups.Count-1 do
      AddGroupToTree(FTemplate.View.Groups[i]);
  finally
    tvView.Items.EndUpdate;
  end;
end;

procedure TfmViewEditor.acNewGroupExecute(Sender: TObject);
var
  Group: TParamGroup;
  Node: TTreeNode;
begin
  Group:=FTemplate.View.Groups.NewItem(FTemplate.View.Groups.GetUniqueName(sNewViewGroupName));
  Node:=AddGroupToTree(Group);
  Node.EditText;
end;

procedure TfmViewEditor.acDeleteGroupUpdate(Sender: TObject);
begin
  acDeleteGroup.Enabled := IsGroupNode(tvView.Selected);
end;

procedure TfmViewEditor.acDeleteGroupExecute(Sender: TObject);
begin
  DeleteNode(tvView.Selected);
end;

function TfmViewEditor.IsGroupNode(Node: TTreeNode): Boolean;
var
  Group: TParamGroup;
begin
  group:=GetGroupByNode(Node);
  if (group<>nil) and (group.GroupName<>grVisibleParams) then result:=True else result:=False;
end;

function TfmViewEditor.IsParamNode(Node: TTreeNode): Boolean;
begin
  Result:=(Node<>nil) and (Node.Data<>nil) and (TObject(Node.Data) is TParam);
end;

function TfmViewEditor.GetParamByNode(Node: TTreeNode;
  var Group: TParamGroup): TParam;
begin
  result:=nil;
  if IsParamNode(Node) then
  begin
    result:=TParam(Node.Data);
    Group:=GetGroupByNode(Node.Parent);
  end;
end;

function TfmViewEditor.GetGroupByNode(Node: TTreeNode): TParamGroup;
begin
  result:=nil;
  if (Node<>nil) and (Node.Data<>nil) and (TObject(Node.Data) is TParamGroup) then
    result:=TParamGroup(Node.Data);
end;

procedure TfmViewEditor.tvViewEdited(Sender: TObject; Node: TTreeNode;
  var S: String);
var
  Group: TParamGroup;
begin
  Group:=GetGroupByNode(Node);
  Group.GroupName:=S;
end;

procedure TfmViewEditor.tvViewEditing(Sender: TObject; Node: TTreeNode;
  var AllowEdit: Boolean);
begin
  if IsGroupNode(Node) then AllowEdit:=True else AllowEdit:=False;
end;

// tvTemplate drag events
procedure TfmViewEditor.frTemplatetvTemplateMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    FDraggedNode:=frTemplate.tvTemplate.GetNodeAt(X, Y);
    if frTemplate.IsParamNode(FDraggedNode) then (Sender as TWinControl).BeginDrag(False);
  end;
end;

procedure TfmViewEditor.frTemplatetvTemplateDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  if (Source = tvView) and (FDraggedNode<>nil) then Accept:=True else Accept:=False;
end;

procedure TfmViewEditor.frTemplatetvTemplateDragDrop(Sender,
  Source: TObject; X, Y: Integer);
begin
  if (Source = tvView) and (FDraggedNode<>nil) then
    DeleteNode(FDraggedNode); 
end;



// tvView drag events

procedure TfmViewEditor.tvViewDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var
  Node: TTreeNode;
begin
  Accept:=False;
  if (Source = frTemplate.tvTemplate) then
  begin
    Node:=tvView.GetNodeAt(X,Y);
    if (GetGroupByNode(Node)<>nil) and (FDraggedNode<>nil) then
    begin
      Accept:=True;
    end;
  end;
end;

procedure TfmViewEditor.tvViewDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  Node: TTreeNode;
  Group: TParamGroup;
begin
  Node:=tvView.GetNodeAt(X,Y);
  Group:=GetGroupByNode(Node);
  if (Group<>nil) and (FDraggedNode<>nil) then
  begin
    if TObject(FDraggedNode.Data) is TParam then
    begin
      if Group.NewItem(TParam(FDraggedNode.Data)) then
      begin
        Node:=AddParamToTree(Node, TParam(FDraggedNode.Data));
        Node.MakeVisible;
      end;
    end;
  end;
end;

procedure TfmViewEditor.tvViewMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    FDraggedNode:=tvView.GetNodeAt(X, Y);
    if IsParamNode(FDraggedNode) then (Sender as TWinControl).BeginDrag(False);
  end;
end;

procedure TfmViewEditor.tvViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = vk_Delete) and (tvView.Selected<>nil) then
    DeleteNode(tvView.Selected);
end;

procedure TfmViewEditor.DeleteNode(Node: TTreeNode);
var
  Group: TParamGroup;
  Param: TParam;
begin
  Group:=GetGroupByNode(tvView.Selected);
  if (group<>nil) and (group.GroupName<>grVisibleParams) then
  begin
    FTemplate.View.Groups.Remove(group);
    group.Free;
    Node.Delete;
  end else
  begin
    Param:=GetParamByNode(tvView.Selected, Group);
    if (Param<>nil) and (Group<>nil) then
    begin
      Group.Remove(Param);
      Node.Delete;
    end;
  end;
end;

procedure TfmViewEditor.acGroupUpUpdate(Sender: TObject);
begin
  acGroupUp.Enabled := IsGroupNode(tvView.Selected) and
    (FTemplate.View.Groups.IndexOf(GetGroupByNode(tvView.Selected))<>1);
end;

procedure TfmViewEditor.acGroupDownUpdate(Sender: TObject);
begin
  acGroupDown.Enabled := IsGroupNode(tvView.Selected) and
    (FTemplate.View.Groups.IndexOf(GetGroupByNode(tvView.Selected))<>FTemplate.View.Groups.Count-1);
end;

procedure TfmViewEditor.acGroupUpExecute(Sender: TObject);
var
  Index: Integer;
begin
  Index:=FTemplate.View.Groups.IndexOf(GetGroupByNode(tvView.Selected));
  FTemplate.View.Groups.Exchange(Index, Index-1);
  tvView.Selected.MoveTo(tvView.Selected.getPrevSibling, naInsert);
end;

procedure TfmViewEditor.acGroupDownExecute(Sender: TObject);
var
  Index: Integer;
begin
  Index:=FTemplate.View.Groups.IndexOf(GetGroupByNode(tvView.Selected));
  FTemplate.View.Groups.Exchange(Index, Index+1);
  tvView.Selected.getNextSibling.MoveTo(tvView.Selected, naInsert);
end;

end.
