{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                  About box component                  }
{*******************************************************}
unit WlAbout;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Registry, regstr;
{.$Define UseCompileDate}

type
  TEXAbout = class(TComponent)
  private
    FExtraInfo: string;
    FIcon: TIcon;
    procedure SetIcon(value: TIcon);
  public
    function Execute: boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ExtraInfo: string read FExtraInfo write FExtraInfo;
    property Icon: TIcon read FIcon write SetIcon;
  end;

  TfrmAboutDlg = class(TForm)
    imgIcon: TImage;
    btnOk: TButton;
    lblApplicationName: TLabel;
    lblTargetOS: TLabel;
    lblCopyright: TLabel;
    lblRegistrationMessage: TLabel;
    lblUserName: TLabel;
    lblCompanyName: TLabel;
    Bevel1: TBevel;
    lblPhysicalMemory: TLabel;
    lblPhMem: TLabel;
    lblFileVersion: TLabel;
    lblExtraInfo: TLabel;
    Bevel2: TBevel;
    lblEMail: TLabel;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure lblEMailClick(Sender: TObject);
  end;


implementation
uses VersionInfo, ShellAPI
{$IfDef UseCompileDate}
,Compile_Date
{$ENDIF};

{$R *.DFM}

var
  frmAboutDlg: TfrmAboutDlg;

function GetStringOSVer: string;
var
  VerInfo : TOSVersionInfo;
  Platform: string;
begin
  VerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  GetVersionEx(VerInfo);
  With VerInfo do
  begin
    case dwPlatformId of
      VER_PLATFORM_WIN32s:
      begin
        Platform:='Windows 3.1x with Win32s';
        dwBuildNumber:=LoWord(dwBuildNumber)
      end;
      VER_PLATFORM_WIN32_WINDOWS :
      begin
        Platform:='Windows 95';
        dwBuildNumber:=LoWord(dwBuildNumber)
      end;
      VER_PLATFORM_WIN32_NT: Platform:='Windows NT';
      else Platform:='?'
    end;
    Result:=format ('%s %d.%d (Build %d: %s)',
      [Platform, dwMajorVersion, dwMinorVersion, LOWORD(dwBuildNumber), szCSDVersion]);
  end;
end;

procedure TEXAbout.SetIcon(value: TIcon);
begin
  FIcon.Assign(value);
end;

function TEXAbout.Execute: boolean;
begin
//  if Assigned(FOnExecute) then FOnExecute(Self);
  frmAboutDlg := TfrmAboutDlg.Create(Application);
  try
    with frmAboutDlg do
    begin
      lblApplicationName.Caption := GetFileDescription;
      {$IfDef UseCompileDate}
      lblFileVersion.Caption:=Format('Version %s (%s)',
        [GetFileVersion, DateTimeToStr(compiletime)]);
      {$ELSE}
      lblFileVersion.Caption:='Version '+GetFileVersion;
      {$ENDIF}
      lblCopyright.Caption := GetLegalCopyright;
//      lblProductName.Caption := GetProductName;
//      lblProductVersion.Caption := 'Version '+GetProductVersion;
      lblEmail.Caption := GetEMailFromVersion;
      lblExtraInfo.Caption := ExtraInfo;
      if Self.Icon.Empty then
        imgIcon.Picture.Icon.Assign(Application.Icon)
      else
        imgIcon.Picture.Icon.Assign(Self.Icon);
      Result := frmAboutDlg.ShowModal = mrOk;
    end;
  finally
    frmAboutDlg.Free;
  end;
//  if Assigned(FOnExecuted) then FOnExecuted(Self);
end;

constructor TEXAbout.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIcon := TIcon.Create;
end;

destructor TEXAbout.Destroy;
begin
  FIcon.Free;
  inherited Destroy;
end;

procedure TfrmAboutDlg.FormCreate(Sender: TObject);
var
  Reg: TRegistry;
  MemStat: TMemoryStatus;
  PathSetup: string;
begin
  Reg := TRegistry.Create;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE;
      if Win32Platform = VER_PLATFORM_WIN32_NT then
        PathSetup:='Software\Microsoft\Windows NT\CurrentVersion' else
        PathSetup:='Software\Microsoft\Windows\CurrentVersion';
      OpenKey(PathSetup, True);
      lblUserName.Caption := ReadString(REGSTR_VAL_REGOWNER);
      lblCompanyName.Caption := ReadString(REGSTR_VAL_REGORGANIZATION);
    end;
  finally
    Reg.Free;
  end;
  MemStat.dwLength := SizeOf(TMemoryStatus);
  GlobalMemoryStatus(MemStat);
  lblPhMem.Caption := Format('%d KB',[MemStat.dwTotalPhys div 1024]);
  lblTargetOs.Caption := GetStringOSVer;
end;

procedure TfrmAboutDlg.lblEMailClick(Sender: TObject);
begin
  ShellExecute(0, nil, PChar('mailto:' + lblEMAIL.Caption), nil, nil, SW_NORMAL);
end;

end.

