{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{               Ancestor for all MDI windows            }
{*******************************************************}
unit childwin;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TfmChildWindow = class(TForm)
    SaveDialog1: TSaveDialog;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    fChanged: boolean;
    FHasNoFile: Boolean;
    { Private declarations }
  public
    procedure LoadFromFile(const FileName: string); virtual; abstract;
    procedure SaveToFile(const FileName: string); virtual; abstract;
    procedure NewFile; virtual;
    function DefaultExtension: string; virtual; abstract;
    procedure Save(CustomName: Boolean);
    property DataChanged: boolean read fChanged write FChanged;
    property HasNoFile: Boolean read FHasNoFile write FHasNoFile;
  end;

function GetFilter(Extension: String): string;
  
const
  extTemplate = 'ipt';
  extSession =  'ips';
  extAllFiles = '*';

wm_child_closed = wm_user+$500;

implementation
uses iphconsts;

{$R *.DFM}

{ TfmChildWindow }

function GetFilter(Extension: String): string;
var
  Filter: string;
begin
  if Extension = extTemplate then Filter := fnTemplate else
    if Extension = extSession then Filter := fnSession else
      if Extension = extAllFiles then Filter := fnAllFiles;
  result:=Format('%s|*.%s', [Filter, Extension]);
end;

procedure TfmChildWindow.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TfmChildWindow.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if not FHasNoFile and DataChanged then
  case MessageDlg(Format('Save changes to %s', [ExtractFileName(Caption)]),
    mtConfirmation, mbYesNoCancel, 0) of
    mrYes: Save(False);
    mrCancel: CanClose:=False;
  end;
end;

procedure TfmChildWindow.NewFile;
begin

end;

procedure TfmChildWindow.Save(CustomName: Boolean);
begin
  if FHasNoFile then Exit;
  if CustomName or (Pos(cptNoname, Caption)=1) then
  begin
    SaveDialog1.InitialDir:=ExtractFilePath(ParamStr(0));
    SaveDialog1.Filter:=Format('%s|%s',  [GetFilter(DefaultExtension),
      GetFilter(extAllFiles)]);
    SaveDialog1.DefaultExt:=DefaultExtension;
    if SaveDialog1.Execute then
    begin
      SaveToFile(SaveDialog1.FileName);
      Caption:=SaveDialog1.FileName;
    end;
  end else
    SaveToFile(Caption);
end;

end.
