{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                   Application form                    }
{*******************************************************}
unit Main;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, Menus,
  StdCtrls, Dialogs, Buttons, Messages, ExtCtrls, ComCtrls, StdActns,
  ActnList, ToolWin, ImgList, AppUtils, Placemnt;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileCloseItem: TMenuItem;
    Window1: TMenuItem;
    Help1: TMenuItem;
    N1: TMenuItem;
    FileExitItem: TMenuItem;
    WindowCascadeItem: TMenuItem;
    WindowTileItem: TMenuItem;
    WindowArrangeItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    OpenDialog: TOpenDialog;
    FileSaveItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    Edit1: TMenuItem;
    CutItem: TMenuItem;
    CopyItem: TMenuItem;
    PasteItem: TMenuItem;
    WindowMinimizeItem: TMenuItem;
    StatusBar: TStatusBar;
    ActionList1: TActionList;
    acEditCut: TEditCut;
    acEditCopy: TEditCopy;
    acEditPaste: TEditPaste;
    acFileNewTemplate: TAction;
    acFileSave: TAction;
    acFileExit: TAction;
    acFileOpen: TAction;
    acFileSaveAs: TAction;
    acWindowCascade: TWindowCascade;
    acWindowTileHorizontal: TWindowTileHorizontal;
    acWindowArrangeAll: TWindowArrange;
    acWindowMinimizeAll: TWindowMinimizeAll;
    acHelpAbout: TAction;
    acFileClose: TWindowClose;
    acWindowTileVertical: TWindowTileVertical;
    WindowTileItem2: TMenuItem;
    ToolBar2: TToolBar;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton9: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton10: TToolButton;
    ToolButton11: TToolButton;
    ImageList1: TImageList;
    acTemplates: TAction;
    acFileNewSession: TAction;
    N2: TMenuItem;
    Newsession1: TMenuItem;
    FormPlacement1: TFormPlacement;
    procedure acFileNewTemplateExecute(Sender: TObject);
    procedure acFileOpenExecute(Sender: TObject);
    procedure acFileExitExecute(Sender: TObject);
    procedure acFileNewSessionExecute(Sender: TObject);
    procedure acFileSaveAsExecute(Sender: TObject);
    procedure acFileSaveExecute(Sender: TObject);
    procedure acFileSaveUpdate(Sender: TObject);
    procedure acFileSaveAsUpdate(Sender: TObject);
    procedure acFileCloseUpdate(Sender: TObject);
    procedure acHelpAboutExecute(Sender: TObject);
  private
    { Private declarations }
    procedure CreateMDIChild(const Name, Extension: string);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
uses iphconsts, ChildWin, TemplateEditor, SessionWindow, wlabout;
{$R *.DFM}

procedure TMainForm.CreateMDIChild(const Name, Extension: string);
var
  Child: TfmChildWindow;
begin
  if Extension = extTemplate then
    Child := TfmTemplateEditor.Create(Application) else
    Child := TfmSessionWindow.Create(Application);
  Child.Caption := Name;
  if FileExists(Name) then Child.LoadFromFile(Name) else Child.NewFile;
end;

procedure TMainForm.acFileNewTemplateExecute(Sender: TObject);
begin
  CreateMDIChild(cptNoname + IntToStr(MDIChildCount + 1), extTemplate);
end;

procedure TMainForm.acFileNewSessionExecute(Sender: TObject);
begin
  CreateMDIChild(cptNoname + IntToStr(MDIChildCount + 1), extSession);
end;

procedure TMainForm.acFileOpenExecute(Sender: TObject);
var
  Extension: string;
begin
  OpenDialog.Filter:=Format('%s|%s|%s',
    [GetFilter(extSession), GetFilter(extTemplate), GetFilter(extAllFiles)]);
  OpenDialog.InitialDir:=ExtractFilePath(ParamStr(0));
  if OpenDialog.Execute then
  begin
    Extension := Copy(ExtractFileExt(OpenDialog.FileName), 2, 3);
    if (UpperCase(Extension)<>extTemplate) and
      (UpperCase(Extension)<>UpperCase(extSession)) then
    case OpenDialog.FilterIndex of
      2: Extension:=extTemplate;
      else Extension:=extSession;
    end;
    CreateMDIChild(OpenDialog.FileName, Extension);
  end;
end;

procedure TMainForm.acFileExitExecute(Sender: TObject);
begin
  Close;
end;

function OnGetDefaultIniName: string;
begin
  result:=ChangeFileExt(ParamStr(0), '.ini');
end;

procedure TMainForm.acFileSaveAsExecute(Sender: TObject);
begin
  TfmChildWindow(ActiveMDIChild).Save(True);
end;

procedure TMainForm.acFileSaveExecute(Sender: TObject);
begin
  TfmChildWindow(ActiveMDIChild).Save(False);
end;

procedure TMainForm.acFileSaveUpdate(Sender: TObject);
begin
  acFileSave.Enabled:=ActiveMDIChild<>nil;
end;

procedure TMainForm.acFileSaveAsUpdate(Sender: TObject);
begin
  acFileSaveAs.Enabled:=ActiveMDIChild<>nil;
end;

procedure TMainForm.acFileCloseUpdate(Sender: TObject);
begin
  acFileClose.Enabled:=ActiveMDIChild<>nil;
end;

procedure TMainForm.acHelpAboutExecute(Sender: TObject);
begin
  With TExAbout.Create(Self) do
  try
    Execute;
  finally
    Free;
  end;
end;

initialization
  apputils.OnGetDefaultIniName:=OnGetDefaultIniName;
end.
