{*******************************************************}
{          Copyright (c) 2000 Vlad Korshunov            }
{                   IPH tools project                   }
{                 Misc utility functions                }
{*******************************************************}
unit utils;
interface
uses windows, sysutils, classes;

procedure RaiseLastWin32Error(const Op: string);
procedure StreamWriteStr(Stream: TStream; Str: String);
procedure WriteStrLevel(Stream: TStream; Level: Integer; Str: String);
function BindProcs(HLibrary: THandle; Vars: Array of Pointer;
  Names: Array of PChar; RaiseException: Boolean = True): Boolean;
function GetModuleName(HLibrary: THandle): String;
function CopyStringToBuf(S: String; Buffer: Pointer; var BufSize: Integer): Boolean;
function SuccessStr(Success: Boolean): String;


const
  seperator = ';';
  TabChar = ' ';
  EndOfLine = #13#10;
  tkComment = '//';

implementation
uses iphconsts;

procedure RaiseLastWin32Error(const Op: string);
var
  LastError: DWORD;
  Error: EWin32Error;
begin
  LastError := GetLastError;
  if LastError <> ERROR_SUCCESS then
    Error := EWin32Error.CreateFmt(SWin32Error, [LastError,
      SysErrorMessage(LastError), Op])
  else
    Error := EWin32Error.CreateFmt(SUnkWin32Error, [OP]);
  Error.ErrorCode := LastError;
  raise Error;
end;

procedure StreamWriteStr(Stream: TStream; Str: String);
begin
  Stream.Write(Pointer(Str)^, Length(Str));
end;

procedure WriteStrLevel(Stream: TStream; Level: Integer; Str: String);
var
  SWrite: String;
  k: integer;
begin
  SWrite := '';
  for k := 0 to Level - 1 do SWrite := SWrite + TabChar;
  SWrite:=SWrite+Str+EndOfLine;
  StreamWriteStr(Stream, SWrite);
end;

function GetModuleName(HLibrary: THandle): String;
var
  FileName: array[0..255] of Char;
begin
  GetModuleFileName(HLibrary, FileName, Sizeof(FileName));
  Result := FileName;
end;

function BindProcs(HLibrary: THandle; Vars: Array of Pointer;
  Names: Array of PChar; RaiseException: Boolean = True): Boolean;
var
  i: Integer;
begin
  result:=True;
  for i:=Low(Vars) to High(Vars) do
  begin
    Pointer(Vars[i]^):=GetProcAddress(HLibrary, Names[i]);
    if (Pointer(Vars[i]^) = nil) and RaiseException then
      raise Exception.CreateFmt(sErrorBindProcedure,
        [Names[i], GetModuleName(HLibrary), SysErrorMessage(GetLastError)]);
  end;
end;

function CopyStringToBuf(S: String; Buffer: Pointer; var BufSize: Integer): Boolean;
var
  L: Integer;
begin
  result:=False;
  L:=Length(S);
  if BufSize>=L then
  begin
    if L<>0 then Move(S[1], Buffer^, L);
    result:=True;
  end;
  BufSize:=L;
end;

function SuccessStr(Success: Boolean): String;
begin
  if Success then result:=sSuccess else result:=sFailed;
end;

end.
